# Copyright (C) 2006-2010 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""UI tests for the test framework."""

import os

from breezy import tests
from breezy.tests import features
from breezy.transport import memory


class SelfTestPatch:

    def get_params_passed_to_core(self, cmdline):
        params = []

        def selftest(*args, **kwargs):
            """Capture the arguments selftest was run with."""
            params.append((args, kwargs))
            return True
        # Yes this prevents using threads to run the test suite in parallel,
        # however we don't have a clean dependency injector for commands,
        # and even if we did - we'd still be testing that the glue is wired
        # up correctly. XXX: TODO: Solve this testing problem.
        original_selftest = tests.selftest
        tests.selftest = selftest
        try:
            self.run_bzr(cmdline)
            return params[0]
        finally:
            tests.selftest = original_selftest


class TestOptions(tests.TestCase, SelfTestPatch):

    def test_load_list(self):
        params = self.get_params_passed_to_core('selftest --load-list foo')
        self.assertEqual('foo', params[1]['load_list'])

    def test_transport_set_to_sftp(self):
        # Test that we can pass a transport to the selftest core - sftp
        # version.
        self.requireFeature(features.paramiko)
        from breezy.tests import stub_sftp
        params = self.get_params_passed_to_core('selftest --transport=sftp')
        self.assertEqual(stub_sftp.SFTPAbsoluteServer,
                         params[1]["transport"])

    def test_transport_set_to_memory(self):
        # Test that we can pass a transport to the selftest core - memory
        # version.
        params = self.get_params_passed_to_core('selftest --transport=memory')
        self.assertEqual(memory.MemoryServer, params[1]["transport"])

    def test_parameters_passed_to_core(self):
        params = self.get_params_passed_to_core('selftest --list-only')
        self.assertTrue("list_only" in params[1])
        params = self.get_params_passed_to_core(
            'selftest --list-only selftest')
        self.assertTrue("list_only" in params[1])
        params = self.get_params_passed_to_core(['selftest', '--list-only',
                                                 '--exclude', 'selftest'])
        self.assertTrue("list_only" in params[1])
        params = self.get_params_passed_to_core(['selftest', '--list-only',
                                                 'selftest', '--randomize', 'now'])
        self.assertSubset(["list_only", "random_seed"], params[1])

    def test_starting_with(self):
        params = self.get_params_passed_to_core('selftest --starting-with foo')
        self.assertEqual(['foo'], params[1]['starting_with'])

    def test_starting_with_multiple_argument(self):
        params = self.get_params_passed_to_core(
            'selftest --starting-with foo --starting-with bar')
        self.assertEqual(['foo', 'bar'], params[1]['starting_with'])

    def test_subunitv1(self):
        self.requireFeature(features.subunit)
        params = self.get_params_passed_to_core('selftest --subunit1')
        self.assertEqual(tests.SubUnitBzrRunnerv1, params[1]['runner_class'])

    def test_subunitv2(self):
        self.requireFeature(features.subunit)
        params = self.get_params_passed_to_core('selftest --subunit2')
        self.assertEqual(tests.SubUnitBzrRunnerv2, params[1]['runner_class'])

    def _parse_test_list(self, lines, newlines_in_header=0):
        "Parse a list of lines into a tuple of 3 lists (header,body,footer)."
        in_header = newlines_in_header != 0
        in_footer = False
        header = []
        body = []
        footer = []
        header_newlines_found = 0
        for line in lines:
            if in_header:
                if line == '':
                    header_newlines_found += 1
                    if header_newlines_found >= newlines_in_header:
                        in_header = False
                        continue
                header.append(line)
            elif not in_footer:
                if line.startswith('-------'):
                    in_footer = True
                else:
                    body.append(line)
            else:
                footer.append(line)
        # If the last body line is blank, drop it off the list
        if len(body) > 0 and body[-1] == '':
            body.pop()
        return (header, body, footer)

    def test_list_only(self):
        # check that brz selftest --list-only outputs no ui noise
        def selftest(*args, **kwargs):
            """Capture the arguments selftest was run with."""
            return True

        def outputs_nothing(cmdline):
            out, err = self.run_bzr(cmdline)
            (header, body, footer) = self._parse_test_list(out.splitlines())
            num_tests = len(body)
            self.assertLength(0, header)
            self.assertLength(0, footer)
            self.assertEqual('', err)
        # Yes this prevents using threads to run the test suite in parallel,
        # however we don't have a clean dependency injector for commands,
        # and even if we did - we'd still be testing that the glue is wired
        # up correctly. XXX: TODO: Solve this testing problem.
        original_selftest = tests.selftest
        tests.selftest = selftest
        try:
            outputs_nothing('selftest --list-only')
            outputs_nothing('selftest --list-only selftest')
            outputs_nothing(
                ['selftest', '--list-only', '--exclude', 'selftest'])
        finally:
            tests.selftest = original_selftest

    def test_lsprof_tests(self):
        params = self.get_params_passed_to_core('selftest --lsprof-tests')
        self.assertEqual(True, params[1]["lsprof_tests"])

    def test_parallel_fork_unsupported(self):
        if getattr(os, "fork", None) is not None:
            self.addCleanup(setattr, os, "fork", os.fork)
            del os.fork
        out, err = self.run_bzr(["selftest", "--parallel=fork", "-s", "bt.x"],
                                retcode=3)
        self.assertIn("platform does not support fork", err)
        self.assertFalse(out)



# Copyright (C) 2008-2012 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""UI helper for the push command."""

from . import branch as _mod_branch
from . import controldir, errors
from . import revision as _mod_revision
from . import transport
from .i18n import gettext
from .trace import note, warning


class PushResult:
    """Result of a push operation.

    :ivar branch_push_result: Result of a push between branches
    :ivar target_branch: The target branch
    :ivar stacked_on: URL of the branch on which the result is stacked
    :ivar workingtree_updated: Whether or not the target workingtree was updated.
    """

    def __init__(self):
        self.branch_push_result = None
        self.stacked_on = None
        self.workingtree_updated = None
        self.target_branch = None

    def report(self, to_file):
        """Write a human-readable description of the result."""
        if self.branch_push_result is None:
            if self.stacked_on is not None:
                note(gettext('Created new stacked branch referring to %s.') %
                     self.stacked_on)
            else:
                note(gettext('Created new branch.'))
        else:
            self.branch_push_result.report(to_file)


def _show_push_branch(br_from, revision_id, location, to_file, verbose=False,
                      overwrite=False, remember=False, stacked_on=None, create_prefix=False,
                      use_existing_dir=False, no_tree=False, lossy=False):
    """Push a branch to a location.

    :param br_from: the source branch
    :param revision_id: the revision-id to push up to
    :param location: the url of the destination
    :param to_file: the output stream
    :param verbose: if True, display more output than normal
    :param overwrite: list of things to overwrite ("history", "tags")
        or boolean indicating for everything
    :param remember: if True, store the location as the push location for
        the source branch
    :param stacked_on: the url of the branch, if any, to stack on;
        if set, only the revisions not in that branch are pushed
    :param create_prefix: if True, create the necessary parent directories
        at the destination if they don't already exist
    :param use_existing_dir: if True, proceed even if the destination
        directory exists without a current control directory in it
    :param lossy: Allow lossy push
    """
    to_transport = transport.get_transport(location, purpose='write')
    try:
        dir_to = controldir.ControlDir.open_from_transport(to_transport)
    except errors.NotBranchError:
        # Didn't find anything
        dir_to = None

    if dir_to is None:
        try:
            br_to = br_from.create_clone_on_transport(to_transport,
                                                      revision_id=revision_id, stacked_on=stacked_on,
                                                      create_prefix=create_prefix, use_existing_dir=use_existing_dir,
                                                      no_tree=no_tree)
        except errors.AlreadyControlDirError:
            raise errors.CommandError(gettext(
                "Target directory %s already contains a .bzr directory, "
                "but it is not valid.") % (location,))
        except transport.FileExists:
            if not use_existing_dir:
                raise errors.CommandError(gettext("Target directory %s"
                                                  " already exists, but does not have a .bzr"
                                                  " directory. Supply --use-existing-dir to push"
                                                  " there anyway.") % location)
            # This shouldn't occur, but if it does the FileExists error will be
            # more informative than an UnboundLocalError for br_to.
            raise
        except transport.NoSuchFile:
            if not create_prefix:
                raise errors.CommandError(gettext("Parent directory of %s"
                                                  " does not exist."
                                                  "\nYou may supply --create-prefix to create all"
                                                  " leading parent directories.")
                                          % location)
            # This shouldn't occur (because create_prefix is true, so
            # create_clone_on_transport should be catching NoSuchFile and
            # creating the missing directories) but if it does the original
            # NoSuchFile error will be more informative than an
            # UnboundLocalError for br_to.
            raise
        except errors.TooManyRedirections:
            raise errors.CommandError(gettext("Too many redirections trying "
                                              "to make %s.") % location)
        push_result = PushResult()
        # TODO: Some more useful message about what was copied
        try:
            push_result.stacked_on = br_to.get_stacked_on_url()
        except (_mod_branch.UnstackableBranchFormat,
                errors.UnstackableRepositoryFormat,
                errors.NotStacked):
            push_result.stacked_on = None
        push_result.target_branch = br_to
        push_result.old_revid = _mod_revision.NULL_REVISION
        push_result.old_revno = 0
        # Remembers if asked explicitly or no previous location is set
        if (remember
                or (remember is None and br_from.get_push_location() is None)):
            # FIXME: Should be done only if we succeed ? -- vila 2012-01-18
            br_from.set_push_location(br_to.base)
    else:
        if stacked_on is not None:
            warning("Ignoring request for a stacked branch as repository "
                    "already exists at the destination location.")
        try:
            push_result = dir_to.push_branch(br_from, revision_id, overwrite,
                                             remember, create_prefix, lossy=lossy)
        except errors.DivergedBranches:
            raise errors.CommandError(gettext('These branches have diverged.'
                                              '  See "brz help diverged-branches"'
                                              ' for more information.'))
        except errors.NoRoundtrippingSupport as e:
            raise errors.CommandError(
                gettext("It is not possible to losslessly "
                        "push to %s. You may want to use --lossy.") %
                e.target_branch.mapping.vcs.abbreviation)
        except errors.NoRepositoryPresent:
            # we have a controldir but no branch or repository
            # XXX: Figure out what to do other than complain.
            raise errors.CommandError(gettext("At %s you have a valid .bzr"
                                              " control directory, but not a branch or repository. This"
                                              " is an unsupported configuration. Please move the target"
                                              " directory out of the way and try again.") % location)
        if push_result.workingtree_updated is False:
            warning("This transport does not update the working "
                    "tree of: %s. See 'brz help working-trees' for "
                    "more information." % push_result.target_branch.base)
    push_result.report(to_file)
    if verbose:
        br_to = push_result.target_branch
        with br_to.lock_read():
            from .log import show_branch_change
            show_branch_change(br_to, to_file, push_result.old_revno,
                               push_result.old_revid)



#
# Copyright (C) 2007 Lukáš Lalinský <lalinsky@gmail.com>
# Copyright (C) 2007,2009 Alexander Belchenko <bialix@ukr.net>
# Copyright (C) 2011 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

# This module is copied from Bazaar Explorer and modified for brz.

"""i18n and l10n support for Bazaar."""

import gettext as _gettext
import os
import sys

_translations = None


def gettext(message):
    """Translate message.

    :returns: translated message as unicode.
    """
    install()
    try:
        return _translations.ugettext(message)
    except AttributeError:
        return _translations.gettext(message)


def ngettext(singular, plural, number):
    """Translate message with plural forms based on `number`.

    :param singular: English language message in singular form
    :param plural: English language message in plural form
    :param number: the number this message should be translated for

    :returns: translated message as unicode.
    """
    install()
    try:
        return _translations.ungettext(singular, plural, number)
    except AttributeError:
        return _translations.ngettext(singular, plural, number)


def N_(msg):
    """Mark message for translation but don't translate it right away."""
    return msg


def gettext_per_paragraph(message):
    """Translate message per paragraph.

    :returns: concatenated translated message as unicode.
    """
    install()
    paragraphs = message.split('\n\n')
    # Be careful not to translate the empty string -- it holds the
    # meta data of the .po file.
    return '\n\n'.join(gettext(p) if p else '' for p in paragraphs)


def disable_i18n():
    """Do not allow i18n to be enabled.  Useful for third party users
    of breezy."""
    global _translations
    _translations = _gettext.NullTranslations()


def installed():
    """Returns whether translations are in use or not."""
    return _translations is not None


def install(lang=None):
    """Enables gettext translations in brz."""
    global _translations
    if installed():
        return
    _translations = install_translations(lang)


def install_translations(lang=None, domain='brz', locale_base=None):
    """Create a gettext translation object.

    :param lang: language to install.
    :param domain: translation domain to install.
    :param locale_base: plugins can specify their own directory.

    :returns: a gettext translations object to use
    """
    if lang is None:
        lang = _get_current_locale()
    if lang is not None:
        languages = lang.split(':')
    else:
        languages = None
    translation = _gettext.translation(
        domain,
        localedir=_get_locale_dir(locale_base),
        languages=languages,
        fallback=True)
    return translation


def add_fallback(fallback):
    """
    Add a fallback translations object.  Typically used by plugins.

    :param fallback: gettext.GNUTranslations object
    """
    install()
    _translations.add_fallback(fallback)


def uninstall():
    """Disables gettext translations."""
    global _translations
    _translations = None


def _get_locale_dir(base):
    """Returns directory to find .mo translations file in, either local or system

    :param base: plugins can specify their own local directory
    """
    if getattr(sys, 'frozen', False):
        if base is None:
            base = os.path.dirname(sys.executable)
        return os.path.join(base, 'locale')
    else:
        if base is None:
            base = os.path.dirname(__file__)
        dirpath = os.path.realpath(os.path.join(base, 'locale'))
        if os.path.exists(dirpath):
            return dirpath
    return os.path.join(sys.prefix, "share", "locale")


def _check_win32_locale():
    for i in ('LANGUAGE', 'LC_ALL', 'LC_MESSAGES', 'LANG'):
        if os.environ.get(i):
            break
    else:
        lang = None
        import locale
        try:
            import ctypes
        except ImportError:
            # use only user's default locale
            lang = locale.getdefaultlocale()[0]
        else:
            # using ctypes to determine all locales
            lcid_user = ctypes.windll.kernel32.GetUserDefaultLCID()
            lcid_system = ctypes.windll.kernel32.GetSystemDefaultLCID()
            if lcid_user != lcid_system:
                lcid = [lcid_user, lcid_system]
            else:
                lcid = [lcid_user]
            lang = [locale.windows_locale.get(i) for i in lcid]
            lang = ':'.join([i for i in lang if i])
        # set lang code for gettext
        if lang:
            os.environ['LANGUAGE'] = lang


def _get_current_locale():
    if not os.environ.get('LANGUAGE'):
        from . import config
        lang = config.GlobalStack().get('language')
        if lang:
            os.environ['LANGUAGE'] = lang
            return lang
    if sys.platform == 'win32':
        _check_win32_locale()
    for i in ('LANGUAGE', 'LC_ALL', 'LC_MESSAGES', 'LANG'):
        lang = os.environ.get(i)
        if lang:
            return lang
    return None


def load_plugin_translations(domain):
    """Load the translations for a specific plugin.

    :param domain: Gettext domain name (usually 'brz-PLUGINNAME')
    """
    locale_base = os.path.dirname(__file__)
    translation = install_translations(domain=domain,
                                       locale_base=locale_base)
    add_fallback(translation)
    return translation



# Copyright (C) 2010 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA


"""Black-box tests for brz rmbranch."""

from breezy import controldir
from breezy.tests import TestCaseWithTransport


class TestRemoveBranch(TestCaseWithTransport):

    def example_tree(self, path='.', format=None):
        tree = self.make_branch_and_tree(path, format=format)
        self.build_tree_contents([(path + '/hello', b'foo')])
        tree.add('hello')
        tree.commit(message='setup')
        self.build_tree_contents([(path + '/goodbye', b'baz')])
        tree.add('goodbye')
        tree.commit(message='setup')
        return tree

    def test_remove_local(self):
        # Remove a local branch.
        tree = self.example_tree('a')
        self.run_bzr_error(['Branch is active. Use --force to remove it.\n'],
                           'rmbranch a')
        self.run_bzr('rmbranch --force a')
        dir = controldir.ControlDir.open('a')
        self.assertFalse(dir.has_branch())
        self.assertPathExists('a/hello')
        self.assertPathExists('a/goodbye')

    def test_no_branch(self):
        # No branch in the current directory.
        self.make_repository('a')
        self.run_bzr_error(['Not a branch'], 'rmbranch a')

    def test_no_tree(self):
        # removing the active branch is possible if there is no tree
        tree = self.example_tree('a')
        tree.controldir.destroy_workingtree()
        self.run_bzr('rmbranch', working_dir='a')
        dir = controldir.ControlDir.open('a')
        self.assertFalse(dir.has_branch())

    def test_no_arg(self):
        # location argument defaults to current directory
        self.example_tree('a')
        self.run_bzr_error(['Branch is active. Use --force to remove it.\n'],
                           'rmbranch a')
        self.run_bzr('rmbranch --force', working_dir='a')
        dir = controldir.ControlDir.open('a')
        self.assertFalse(dir.has_branch())

    def test_remove_colo(self):
        # Remove a colocated branch.
        tree = self.example_tree('a')
        tree.controldir.create_branch(name="otherbranch")
        self.assertTrue(tree.controldir.has_branch('otherbranch'))
        self.run_bzr('rmbranch %s,branch=otherbranch' %
                     tree.controldir.user_url)
        dir = controldir.ControlDir.open('a')
        self.assertFalse(dir.has_branch('otherbranch'))
        self.assertTrue(dir.has_branch())

    def test_remove_colo_directory(self):
        # Remove a colocated branch.
        tree = self.example_tree('a')
        tree.controldir.create_branch(name="otherbranch")
        self.assertTrue(tree.controldir.has_branch('otherbranch'))
        self.run_bzr('rmbranch otherbranch -d %s' % tree.controldir.user_url)
        dir = controldir.ControlDir.open('a')
        self.assertFalse(dir.has_branch('otherbranch'))
        self.assertTrue(dir.has_branch())

    def test_remove_active_colo_branch(self):
        # Remove a colocated branch.
        dir = self.make_repository('a').controldir
        branch = dir.create_branch('otherbranch')
        branch.create_checkout('a')
        self.run_bzr_error(['Branch is active. Use --force to remove it.\n'],
                           'rmbranch otherbranch -d %s' % branch.controldir.user_url)
        self.assertTrue(dir.has_branch('otherbranch'))
        self.run_bzr('rmbranch --force otherbranch -d %s' %
                     branch.controldir.user_url)
        self.assertFalse(dir.has_branch('otherbranch'))



# Copyright (C) 2011 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA


"""Tests for _make_parents_provider on stacked repositories."""


from breezy.tests.per_repository import TestCaseWithRepository


class Test_MakeParentsProvider(TestCaseWithRepository):

    def test_add_fallback_after_make_pp(self):
        """Fallbacks added after _make_parents_provider are used by that
        provider.
        """
        referring_repo = self.make_repository('repo')
        pp = referring_repo._make_parents_provider()
        # Initially referring_repo has no revisions and no fallbacks
        self.addCleanup(referring_repo.lock_read().unlock)
        self.assertEqual({}, pp.get_parent_map([b'revid2']))
        # Add a fallback repo with a commit
        wt_a = self.make_branch_and_tree('fallback')
        wt_a.commit('first commit', rev_id=b'revid1')
        wt_a.commit('second commit', rev_id=b'revid2')
        fallback_repo = wt_a.branch.repository
        referring_repo.add_fallback_repository(fallback_repo)
        # Now revid1 appears in pp's results.
        self.assertEqual((b'revid1',), pp.get_parent_map(
            [b'revid2'])[b'revid2'])



# Copyright (C) 2006, 2007, 2009, 2012 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from breezy import branch, tests


class TestRevisionHistory(tests.TestCaseWithTransport):

    def _build_branch(self):
        # setup a standalone branch with three commits
        tree = self.make_branch_and_tree('test')
        with open('test/foo', 'wb') as f:
            f.write(b'1111\n')
        tree.add('foo')
        tree.commit('added foo', rev_id=b'revision_1')
        with open('test/foo', 'wb')as f:
            f.write(b'2222\n')
        tree.commit('updated foo', rev_id=b'revision_2')
        with open('test/foo', 'wb')as f:
            f.write(b'3333\n')
        tree.commit('updated foo again', rev_id=b'revision_3')
        return tree

    def _check_revision_history(self, location='', working_dir=None):
        rh = self.run_bzr(['revision-history', location],
                          working_dir=working_dir)[0]
        self.assertEqual(rh, 'revision_1\nrevision_2\nrevision_3\n')

    def test_revision_history(self):
        """No location"""
        self._build_branch()
        self._check_revision_history(working_dir='test')

    def test_revision_history_with_location(self):
        """With a specified location."""
        self._build_branch()
        self._check_revision_history('test')

    def test_revision_history_with_repo_branch(self):
        """With a repository branch location."""
        self._build_branch()
        self.run_bzr('init-shared-repo repo')
        self.run_bzr('branch test repo/test')
        self._check_revision_history('repo/test')

    def test_revision_history_with_checkout(self):
        """With a repository branch checkout location."""
        self._build_branch()
        self.run_bzr('init-shared-repo repo')
        self.run_bzr('branch test repo/test')
        self.run_bzr('checkout repo/test test-checkout')
        self._check_revision_history('test-checkout')

    def test_revision_history_with_lightweight_checkout(self):
        """With a repository branch lightweight checkout location."""
        self._build_branch()
        self.run_bzr('init-shared-repo repo')
        self.run_bzr('branch test repo/test')
        self.run_bzr('checkout --lightweight repo/test test-checkout')
        self._check_revision_history('test-checkout')



# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Tests for WorkingTree.revision_tree.

These tests are in addition to the tests from
per_tree.test_revision_tree which cover the behaviour expected from
all Trees. WorkingTrees implement the revision_tree api to allow access to
cached data, but we don't require that all WorkingTrees have such a cache,
so these tests are testing that when there is a cache, it performs correctly.
"""

from breezy import errors, tests
from breezy import transport as _mod_transport
from breezy.tests import per_workingtree


class TestRevisionTree(per_workingtree.TestCaseWithWorkingTree):

    def test_get_zeroth_basis_tree_via_revision_tree(self):
        tree = self.make_branch_and_tree('.')
        try:
            revision_tree = tree.revision_tree(tree.last_revision())
        except errors.NoSuchRevision:
            # its ok for a working tree to not cache trees, so just return.
            return
        basis_tree = tree.basis_tree()
        self.assertTreesEqual(revision_tree, basis_tree)

    def test_get_nonzeroth_basis_tree_via_revision_tree(self):
        tree = self.make_branch_and_tree('.')
        revision1 = tree.commit('first post')
        revision_tree = tree.revision_tree(revision1)
        basis_tree = tree.basis_tree()
        self.assertTreesEqual(revision_tree, basis_tree)

    def test_get_pending_merge_revision_tree(self):
        tree = self.make_branch_and_tree('tree1')
        tree.commit('first post')
        tree2 = tree.controldir.sprout('tree2').open_workingtree()
        revision1 = tree2.commit('commit in branch', allow_pointless=True)
        tree.merge_from_branch(tree2.branch)
        try:
            cached_revision_tree = tree.revision_tree(revision1)
        except errors.NoSuchRevision:
            # its ok for a working tree to not cache trees, so just return.
            return
        real_revision_tree = tree2.basis_tree()
        self.assertTreesEqual(real_revision_tree, cached_revision_tree)

    def test_get_uncached_basis_via_revision_tree(self):
        # The basis_tree method returns an empty tree when you ask for the
        # basis if the basis is not cached, and it is a ghost. However the
        # revision_tree method should always raise when a request tree is not
        # cached, so we force this by setting a basis that is a ghost and
        # thus cannot be cached.
        tree = self.make_branch_and_tree('.')
        if not tree.branch.repository._format.supports_ghosts:
            self.skipTest('format does not support ghosts')
        tree.set_parent_ids([b'a-ghost'], allow_leftmost_as_ghost=True)
        self.assertRaises(errors.NoSuchRevision,
                          tree.revision_tree, b'a-ghost')

    def test_revision_tree_different_root_id(self):
        """A revision tree might have a very different root."""
        tree = self.make_branch_and_tree('tree1')
        if not tree.supports_setting_file_ids():
            raise tests.TestNotApplicable(
                'tree does not support setting file ids')
        tree.set_root_id(b'one')
        rev1 = tree.commit('first post')
        tree.set_root_id(b'two')
        try:
            cached_revision_tree = tree.revision_tree(rev1)
        except errors.NoSuchRevision:
            # its ok for a working tree to not cache trees, so just return.
            return
        repository_revision_tree = tree.branch.repository.revision_tree(rev1)
        self.assertTreesEqual(repository_revision_tree, cached_revision_tree)


class TestRevisionTreeKind(per_workingtree.TestCaseWithWorkingTree):

    def make_branch_with_merged_deletions(self, relpath='tree'):
        tree = self.make_branch_and_tree(relpath)
        files = ['a', 'b/', 'b/c']
        self.build_tree(files, line_endings='binary',
                        transport=tree.controldir.root_transport)
        tree.add(files)
        base_revid = tree.commit('a, b and b/c')
        tree2 = tree.controldir.sprout(relpath + '2').open_workingtree()
        # Delete 'a' in tree
        tree.remove('a', keep_files=False)
        this_revid = tree.commit('remove a')
        # Delete 'c' in tree2
        tree2.remove('b/c', keep_files=False)
        tree2.remove('b', keep_files=False)
        other_revid = tree2.commit('remove b/c')
        # Merge tree2 into tree
        tree.merge_from_branch(tree2.branch)
        return tree, [base_revid, this_revid, other_revid]

    def test_kind_parent_tree(self):
        tree, [base_revid, this_revid,
               other_revid] = self.make_branch_with_merged_deletions()
        tree.lock_read()
        self.addCleanup(tree.unlock)
        parents = tree.get_parent_ids()
        self.assertEqual([this_revid, other_revid], parents)
        basis = tree.revision_tree(parents[0])
        basis.lock_read()
        self.addCleanup(basis.unlock)
        self.assertRaises(_mod_transport.NoSuchFile, basis.kind, 'a')
        self.assertEqual(['directory', 'file'],
                         [basis.kind('b'), basis.kind('b/c')])
        try:
            other = tree.revision_tree(parents[1])
        except errors.NoSuchRevisionInTree:
            raise tests.TestNotApplicable(
                'Tree type %s caches only the basis revision tree.'
                % type(tree))
        other.lock_read()
        self.addCleanup(other.unlock)
        self.assertRaises(_mod_transport.NoSuchFile, other.kind, 'b')
        self.assertRaises(_mod_transport.NoSuchFile, other.kind, 'c')
        self.assertEqual('file', other.kind('a'))



# Copyright (C) 2008 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Tests for break_lock on a repository with external references."""

from breezy import errors, ui
from breezy.tests.per_repository_reference import \
    TestCaseWithExternalReferenceRepository


class TestBreakLock(TestCaseWithExternalReferenceRepository):

    def test_break_lock(self):
        base = self.make_repository('base')
        repo = self.make_referring('referring', base)
        unused_repo = repo.controldir.open_repository()
        base.lock_write()
        self.addCleanup(base.unlock)
        # break_lock when locked should
        repo.lock_write()
        self.assertEqual(repo.get_physical_lock_status(),
                         unused_repo.get_physical_lock_status())
        if not unused_repo.get_physical_lock_status():
            # 'lock_write' has not taken a physical mutex out.
            repo.unlock()
            return
        ui.ui_factory = ui.CannedInputUIFactory([True])
        unused_repo.break_lock()
        self.assertRaises(errors.LockBroken, repo.unlock)
